@@ -1,4 +1,4 @@
-from re import search as re_search
+from re import search
 from time import perf_counter
 from time import sleep
 from typing import Any
@@ -18,11 +18,15 @@ from .exceptions import CrawlDelayError
 from .exceptions import DisallowedPath
 from .journal import Journal
 from .parse import BeautifulSoup
-from .parse import Tag
 from .parse import check_page
 from .parse import check_page_raise
 from .parse import parse_page
+from .parse import parse_search_submissions
+from .parse import parse_user_favorites
+from .parse import parse_user_journals
+from .parse import parse_user_submissions
 from .parse import parse_watchlist
+from .parse import username_url
 from .submission import Submission
 from .submission import SubmissionPartial
 from .user import User
@@ -30,7 +34,7 @@ from .user import User
 
 class FAAPI:
     def __init__(self, cookies: List[Dict[str, Any]] = None):
-        self.cookies: List[Dict[str, Any]] = [] if cookies is None else cookies
+        self.cookies: List[Dict[str, Any]] = cookies or []
         self.session: CloudflareScraper = make_session(self.cookies)
         self.robots: Dict[str, List[str]] = get_robots()
         self.crawl_delay: float = float(self.robots.get("Crawl-delay", [1.0])[0])
@@ -50,7 +54,7 @@ class FAAPI:
 
     def check_path(self, path: str):
         for pattern in self.robots.get("disallow", []):
-            if re_search(fr"^{pattern.replace('*', '.*')}", "/" + path.lstrip("/")):
+            if search(fr"^{pattern.replace('*', '.*')}", "/" + path.lstrip("/")):
                 raise DisallowedPath(f"Path {path} is not allowed by robots.txt {pattern}")
 
     @property
@@ -83,42 +87,46 @@ class FAAPI:
         return Journal(self.get_parse(join_url("journal", int(journal_id))))
 
     def get_user(self, user: str) -> User:
-        return User(self.get_parse(join_url("user", user)))
+        return User(self.get_parse(join_url("user", username_url(user))))
 
     def gallery(self, user: str, page: int = 1) -> Tuple[List[SubmissionPartial], int]:
-        check_page_raise(page_parsed := self.get_parse(join_url("gallery", user, int(page))))
-        return list(map(SubmissionPartial, s := page_parsed.select("figure[id^='sid-']"))), (page + 1) if s else 0
+        check_page_raise(page_parsed := self.get_parse(join_url("gallery", username_url(user), int(page))))
+        info_parsed: Dict[str, Any] = parse_user_submissions(page_parsed)
+        for s in (submissions := list(map(SubmissionPartial, info_parsed["figures"]))):
+            s.author.status, s.author.user_page = info_parsed["user_status"], info_parsed["user_icon_url"]
+        return submissions, (page + 1) * (not info_parsed["last_page"])
 
     def scraps(self, user: str, page: int = 1) -> Tuple[List[SubmissionPartial], int]:
-        check_page_raise(page_parsed := self.get_parse(join_url("scraps", user, int(page))))
-        return list(map(SubmissionPartial, s := page_parsed.select("figure[id^='sid-']"))), (page + 1) if s else 0
+        check_page_raise(page_parsed := self.get_parse(join_url("scraps", username_url(user), int(page))))
+        info_parsed: Dict[str, Any] = parse_user_submissions(page_parsed)
+        for s in (submissions := list(map(SubmissionPartial, info_parsed["figures"]))):
+            s.author.status, s.author.user_page = info_parsed["user_status"], info_parsed["user_icon_url"]
+        return submissions, (page + 1) * (not info_parsed["last_page"])
 
     def favorites(self, user: str, page: str = "") -> Tuple[List[SubmissionPartial], str]:
-        check_page_raise(page_parsed := self.get_parse(join_url("favorites", user, page.strip())))
-        tag_next: Optional[Tag] = page_parsed.select_one("a[class~=button][class~=standard][class~=right]")
-        next_page: str = tag_next["href"].split("/", 3)[-1] if tag_next else ""
-        return list(map(SubmissionPartial, page_parsed.select("figure[id^='sid-']"))), next_page
+        check_page_raise(page_parsed := self.get_parse(join_url("favorites", username_url(user), page.strip())))
+        info_parsed: Dict[str, Any] = parse_user_favorites(page_parsed)
+        for s in (submissions := list(map(SubmissionPartial, info_parsed["figures"]))):
+            s.author.status, s.author.user_page = info_parsed["user_status"], info_parsed["user_icon_url"]
+        return submissions, info_parsed["next_page"]
 
     def journals(self, user: str, page: int = 1) -> Tuple[List[Journal], int]:
-        check_page_raise(page_parsed := self.get_parse(join_url("journals", user, int(page))))
-        username: str = page_parsed.select_one("div[class~=username] span").text.strip()[1:]
-        journals: List[Journal] = list(map(Journal, page_parsed.select("section[id^='jid:']")))
-        for j in journals:
-            j.author = username
-        return journals, (page + 1) if journals else 0
+        check_page_raise(page_parsed := self.get_parse(join_url("journals", username_url(user), int(page))))
+        info_parsed: Dict[str, Any] = parse_user_journals(page_parsed)
+        for j in (journals := list(map(Journal, info_parsed["sections"]))):
+            j.author.name, j.author.status, j.author.user_icon_url = \
+                info_parsed["user_name"], info_parsed["user_status"], info_parsed["user_icon_url"]
+        return journals, (page + 1) * (not info_parsed["last_page"])
 
     def search(self, q: str, page: int = 1, **params) -> Tuple[List[SubmissionPartial], int, int, int, int]:
         check_page_raise(page_parsed := self.get_parse("search", q=q, page=(page := int(page)), **params))
-        tag_stats: Tag = page_parsed.select_one("div[id='query-stats']")
-        for div in tag_stats.select("div"):
-            div.decompose()
-        a, b, tot = map(int, re_search(r"(\d+)[^\d]*(\d+)[^\d]*(\d+)", tag_stats.text.strip()).groups())
-        next_page: int = (page + 1) if b < tot else 0
-        return list(map(SubmissionPartial, page_parsed.select("figure[id^='sid-']"))), next_page, a - 1, b - 1, tot
+        info_parsed: Dict[str, Any] = parse_search_submissions(page_parsed)
+        return (list(map(SubmissionPartial, info_parsed["figures"])), (page + 1) * (not info_parsed["last_page"]),
+                info_parsed["from"], info_parsed["to"], info_parsed["total"])
 
     def watchlist_to(self, user: str) -> List[User]:
         users: List[User] = []
-        for s, u in parse_watchlist(self.get_parse(join_url("watchlist", "to", user))):
+        for s, u in parse_watchlist(self.get_parse(join_url("watchlist", "to", username_url(user)))):
             user: User = User()
             user.name = u
             user.status = s
@@ -127,7 +135,7 @@ class FAAPI:
 
     def watchlist_by(self, user: str) -> List[User]:
         users: List[User] = []
-        for s, u in parse_watchlist(self.get_parse(join_url("watchlist", "by", user))):
+        for s, u in parse_watchlist(self.get_parse(join_url("watchlist", "by", username_url(user)))):
             user: User = User()
             user.name = u
             user.status = s
@@ -143,7 +151,7 @@ class FAAPI:
         4 request error
         """
 
-        if not (res := self.get(join_url("user", user))).ok:
+        if not (res := self.get(join_url("user", username_url(user)))).ok:
             return 4
         elif (check := check_page(parse_page(res.text))) == 0:
             return 0
