@@ -1,11 +1,14 @@
 from typing import List
 from typing import Optional
 
+from .connection import join_url
+from .connection import root
 from .parse import BeautifulSoup
 from .parse import Tag
 from .parse import check_page_raise
 from .parse import parse_submission_figure
 from .parse import parse_submission_page
+from .user import User
 
 
 class SubmissionPartial:
@@ -16,7 +19,7 @@ class SubmissionPartial:
 
         self.id: int = 0
         self.title: str = ""
-        self.author: str = ""
+        self.author: User = User()
         self.rating: str = ""
         self.type: str = ""
         self.thumbnail_url: str = ""
@@ -26,7 +29,7 @@ class SubmissionPartial:
     def __iter__(self):
         yield "id", self.id
         yield "title", self.title
-        yield "author", self.author
+        yield "author", dict(self.author)
         yield "rating", self.rating
         yield "type", self.type
         yield "thumbnail_url", self.thumbnail_url
@@ -37,9 +40,13 @@ class SubmissionPartial:
     def __str__(self):
         return f"{self.id} {self.author} {self.title}"
 
+    @property
+    def url(self):
+        return join_url(root, "view", self.id)
+
     def parse(self, submission_figure: Tag = None):
         assert submission_figure is None or isinstance(submission_figure, Tag)
-        self.submission_figure = submission_figure if submission_figure is not None else self.submission_figure
+        self.submission_figure = submission_figure or self.submission_figure
         if self.submission_figure is None:
             return
 
@@ -47,7 +54,7 @@ class SubmissionPartial:
 
         self.id: int = parsed["id"]
         self.title: str = parsed["title"]
-        self.author: str = parsed["author"]
+        self.author.name = parsed["author"]
         self.rating: str = parsed["rating"]
         self.type: str = parsed["type"]
         self.thumbnail_url: str = parsed["thumbnail_url"]
@@ -61,7 +68,7 @@ class Submission:
 
         self.id: int = 0
         self.title: str = ""
-        self.author: str = ""
+        self.author: User = User()
         self.date: str = ""
         self.tags: List[str] = []
         self.category: str = ""
@@ -74,14 +81,13 @@ class Submission:
         self.folder: str = ""
         self.file_url: str = ""
         self.thumbnail_url: str = ""
-        self.user_icon_url: str = ""
 
         self.parse()
 
     def __iter__(self):
         yield "id", self.id
         yield "title", self.title
-        yield "author", self.author
+        yield "author", dict(self.author)
         yield "date", self.date
         yield "tags", self.tags
         yield "category", self.category
@@ -94,7 +100,6 @@ class Submission:
         yield "folder", self.folder
         yield "file_url", self.file_url
         yield "thumbnail_url", self.thumbnail_url
-        yield "user_icon_url", self.user_icon_url
 
     def __repr__(self):
         return repr(dict(self))
@@ -102,9 +107,13 @@ class Submission:
     def __str__(self):
         return f"{self.id} {self.author} {self.title}"
 
+    @property
+    def url(self):
+        return join_url(root, "view", self.id)
+
     def parse(self, submission_page: BeautifulSoup = None):
         assert submission_page is None or isinstance(submission_page, BeautifulSoup)
-        self.submission_page = submission_page if submission_page is not None else self.submission_page
+        self.submission_page = submission_page or self.submission_page
         if self.submission_page is None:
             return
 
@@ -114,7 +123,8 @@ class Submission:
 
         self.id: int = parsed["id"]
         self.title: str = parsed["title"]
-        self.author: str = parsed["author"]
+        self.author.name = parsed["author"]
+        self.author.user_icon_url = parsed["user_icon_url"]
         self.date: str = parsed["date"]
         self.tags: List[str] = parsed["tags"]
         self.category: str = parsed["category"]
@@ -127,4 +137,3 @@ class Submission:
         self.folder: str = parsed["folder"]
         self.file_url: str = parsed["file_url"]
         self.thumbnail_url: str = parsed["thumbnail_url"]
-        self.user_icon_url: str = parsed["user_icon_url"]
